//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import NevisMobileAuthentication

struct TypedAccountSelectionContext: Encodable {
	// MARK: Properties

	let wrapped: AccountSelectionContext
	let operationId: String

	// MARK: Encodable

	enum CodingKeys: CodingKey {
		case operationId
		case accounts
		case authenticators
		case transactionConfirmationData
	}

	func encode(to encoder: Encoder) throws {
		var container = encoder.container(keyedBy: CodingKeys.self)
		try container.encode(operationId, forKey: .operationId)
		try container.encode(wrapped.accounts, forKey: .accounts)
		let authenticators = Set(wrapped.authenticators.map { TypedAuthenticator(wrapped: $0) })
		try container.encodeIfPresent(authenticators, forKey: .authenticators)
		if let transactionConfirmationData = wrapped.transactionConfirmationData {
			let decoded = String(decoding: transactionConfirmationData, as: UTF8.self)
			try container.encode(decoded, forKey: .transactionConfirmationData)
		}
	}
}
