//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import NevisMobileAuthentication

struct TypedAuthenticator: Encodable {
	// MARK: Properties

	let wrapped: Authenticator

	// MARK: Encodable

	enum CodingKeys: CodingKey {
		case aaid
		case registration
		case userEnrollment
		case isSupportedByHardware
	}

	func encode(to encoder: Encoder) throws {
		var container = encoder.container(keyedBy: CodingKeys.self)
		try container.encode(wrapped.aaid, forKey: .aaid)
		try container.encodeIfPresent(wrapped.registration, forKey: .registration)
		try container.encode(wrapped.isSupportedByHardware, forKey: .isSupportedByHardware)
		if let userEnrollment = wrapped.userEnrollment {
			let typedUserEnrollment = TypedUserEnrollment(wrapped: userEnrollment)
			try container.encode(typedUserEnrollment, forKey: .userEnrollment)
		}
	}
}

// MARK: - Hashable

extension TypedAuthenticator: Hashable {
	public func hash(into hasher: inout Hasher) {
		wrapped.hash(into: &hasher)
	}

	static func == (lhs: TypedAuthenticator, rhs: TypedAuthenticator) -> Bool {
		lhs.wrapped == rhs.wrapped
	}
}
