//
// Copyright © 2022 Nevis Security AG. All rights reserved.
//

import NevisMobileAuthentication

struct TypedAuthorizationProvider: Codable {
	// MARK: Properties

	let wrapped: AuthorizationProvider

	// MARK: Initialization

	init(wrapped: AuthorizationProvider) {
		self.wrapped = wrapped
	}

	// MARK: Decodable

	init(from decoder: Decoder) throws {
		if let cookieAuthorizationProvider = try? CookieAuthorizationProvider(from: decoder) {
			self.wrapped = cookieAuthorizationProvider
		}
		else if let jwtAuthorizationProvider = try? JwtAuthorizationProvider(from: decoder) {
			self.wrapped = jwtAuthorizationProvider
		}
		else {
			throw PluginError.illegalState("Failed to decode authorization provider!")
		}
	}

	// MARK: Encodable

	enum AuthorizationProviderType: String, Encodable {
		case cookie = "CookieAuthorizationProvider"
		case jwt = "JwtAuthorizationProvider"
	}

	func encode(to encoder: Encoder) throws {
		var container = encoder.container(keyedBy: TypedCodingKeys.self)
		switch wrapped {
		case is CookieAuthorizationProvider:
			try container.encode(AuthorizationProviderType.cookie, forKey: .type)
		case is JwtAuthorizationProvider:
			try container.encode(AuthorizationProviderType.jwt, forKey: .type)
		default:
			throw PluginError.illegalState("Unsupported authorization provider!")
		}

		try wrapped.encode(to: container.superEncoder(forKey: .data))
	}
}
